#######################################################################################################
# Function to calculate good starting values for the sech-square model
# Amplitude rescaled to be per 100 population
#######################################################################################################

InitSech2Fun <- function(mCall, data, LHS) {
  
  data <- as.data.frame(data)
  pop <- data$pop
  
  # Fit a series of nls model to get very good starting values
  npars <- 5e2 # No of nls fits
  
  # Hypercube of starting values
  phase_min <- -25
  phase_max <- 25
  sig_max <- 15
  start <- sobolDesign(lower = c(P = phase_min, S = 0), 
                       upper = c(P = phase_max, S = sig_max), 
                       nseq = npars)
  start$A <- max(data$no) # Amplitude 
  
  # Run npars estimations from different starting values
  # Upper bound for sigma fixed to 10; other parameters are bounded within their natural range
  fit1 <- list(npars)
  for(i in 1:nrow(start)) {
    start_cur <- setNames(object = as.numeric(start[i, ]), nm = colnames(start))
    
    # Fit complex model
    fit1[[i]] <- try(
      nls(formula = no ~ A * 4 / (exp(-(x - P) / S) + exp((x - P) / S)) ^ 2, 
          data = as.data.frame(data), 
          start = list(A = start_cur["A"], P = start_cur["P"], S = start_cur["S"]),
          algorithm = "port", 
          lower = c(0, phase_min, 0), 
          upper = c(Inf, phase_max, Inf))
    )
  }
  fit1 <- fit1[sapply(fit1, class) != "try-error"] # Remove estimations that failed
  
  if(length(fit1) >= 1) {
    fit1 <- fit1[[which.max(sapply(fit1, logLik))]] # Keep the estimation with highest log-likelihood
    est <- coef(fit1)
    est["A"] <- 1e2 * est["A"] / mean(pop) # Per capita amplitude per 100
    
  } else {
    est <- rep(NA, 3)
  }
  
  out <- setNames(object = unname(est), nm = mCall[c("amp", "ph", "sig")])
  return(out)
}