#######################################################################################################
# Fit the sech-squared NLME model to the ILI region-level data in France during 2000/01-2013/14
#######################################################################################################

rm(list =  ls())

# Install required packages, if necessary ---------------------------------
library(checkpoint)
checkpoint(snapshotDate = "2016-12-01", scanForPackages = F)
print(.libPaths())

library(plyr)
library(reshape2)
library(ggplot2)
library(magrittr)
library(nlme)
library(pomp)
source("InitSech2Fun.R") # Self-starting sech-squared model
theme_set(theme_bw())

# Load ILI data and plot them---------------------------------------------------------------
dat <- readRDS(file = "ILIdat.rds")
print(summary(dat))
print(levels(dat$epiyear))
print(levels(dat$area))

# Plot time series
pl <- ggplot(data = dat, mapping = aes(x = week_no_cont, y = 1e2 * no / pop)) + 
  geom_line() + facet_grid(area ~ epiyear, scales = "fixed") + 
  labs(x = "Week no (centered on last week of calendar year)", y = "Weekly incidence (per 100)")
print(pl)

# Create a self-starting sech-squared model ----------------------------------
dat$x <- dat$week_no_cont

SS_SechFun <- selfStart(model = ~ 1e-2 * pop * amp * 4 / (exp(-(x - ph) / sig) + exp((x - ph) / sig)) ^ 2,
                        initial = InitSech2Fun, 
                        parameters = c("amp", "ph", "sig"), 
                        template = function(x, pop, amp, ph, sig){})

# Test on one group
test <- F
if(test) {
  dat_sub <- dat %>% subset(group == "South East_2011")
  
  # Plot the data
  plot(dat_sub$x, 1e2 * dat_sub$no / dat_sub$pop, 
       type = "l", xlab = "Time (weeks)", 
       ylab = "Weekly cases per 100", las = 1, bty = "l")
  
  # Estimate parameters in that group
  theta <- getInitial(object = no ~ SS_SechFun(x, pop, amp, ph, sig), 
                      data = dat_sub)
  print(theta)
  
  # Superimpose model fit
  yy <- SS_SechFun(x = dat_sub$week_no_cont, pop = dat_sub$pop,
                   amp = theta["amp"], ph = theta["ph"], sig = theta["sig"])
  lines(dat_sub$x, 1e2 * yy / dat_sub$pop, col = "blue")
}

# Create a grouped data object at the region-year level --------------------------------------------
dat_g <- groupedData(formula = no ~ x | group, data = dat)
n_area <- nlevels(dat$area) # No of regions (5)
n_years <- nlevels(dat$epiyear) # No of epiyears (14)

# Use the nlsList function to estimate parameters separately for each group ------------------------
# These values will be used to initialize the estimation of the NLME model
# The results were already recorded using the function bake
# To re-run, delete the "fit_nls.rds" file and execute this code section
# The execution time was about 4 min on MacBook Pro 3,1 GHz Intel Core i7
tic <- Sys.time()
fit_nls <- bake(file = "saved_objects/fit_nls.rds", 
                expr = nlsList(model = no ~ SS_SechFun(x, pop, amp, ph, sig), 
                               data = dat_g))
toc <- Sys.time()
print(toc - tic)

# Examine parameter estimates
print(summary(coef(fit_nls)))
pairs(fit_nls)
plot(intervals(fit_nls))

# Initial NLME fit ---------------------------------------------------------------
# General variance covariance matrix for the random effects; single fixed-effect for each parameter
# Execution time: approx. 10s 

# Modify control parameters of the algorithm
control_list <- nlmeControl()
control_list$maxIter <- 100 # Increase the max no of iterations 
control_list$returnObject <- TRUE

tic <- Sys.time()
fit_nlme0 <- bake(file = "saved_objects/fit_nlme0.rds", 
                  expr = {
                    nlme(model = fit_nls, 
                         data = dat_g, 
                         control = control_list)
                  })
toc <- Sys.time()
print(toc - tic)

# Display model estimates
print(fit_nlme0)

# Add year as covariates for the peak-time fixed-effect ---------------------------------------------------
# This model is the final ILI model, whose results are presented in the main text
modFix <- fixef(fit_nlme0) # Fixed-effects estimates
fit_re <- ranef(fit_nlme0, augFrame = T, data = dat_g %>% subset(select = c(area, epiyear))) # Random-effects
fix0 <- ddply(fit_re, ~ epiyear, .fun = function(x) mean(x[, "ph"])) # Average random-effects across regions

fit_nlme1 <- bake(file = "saved_objects/fit_nlme1.rds", 
                  expr = {
                    update(fit_nlme0,
                           fixed = list(amp ~ 1, ph ~ epiyear, sig ~ 1),
                           start = unname(
                             c(
                               modFix["amp"], 
                               modFix["ph"] + fix0$V1[1],
                               fix0$V1[-1] - fix0$V1[1],
                               modFix["sig"]
                             )
                           ),
                           control = control_list
                    )
                  })
print(anova(fit_nlme0, fit_nlme1))
print(fit_nlme1)

#######################################################################################################
# End
#######################################################################################################